----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    22:52:09 04/15/2010 
-- Design Name: 
-- Module Name:    ClockSync - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ClockSync is
    Port ( slowaccessout : out  STD_LOGIC;
			  fakeclk : out std_logic;
			  cpuclk : in std_logic;
           startfast : in  STD_LOGIC;
           extclkdel : in  STD_LOGIC;
           extclk : in  STD_LOGIC;
			  rdy : in std_logic;
			  reset : in std_logic;
			  isslow : in std_logic
		);
end ClockSync;

architecture Behavioral of ClockSync is

   signal slowaccess : std_logic := '0';
	signal slowdetected : std_logic := '0';
	signal slowdone : std_logic := '0';
	
begin

		slowaccessout <= slowaccess;
		
		-- slow clock handling
		DoSlowAccess: process(extclkdel, startfast, slowdetected)
		begin
			if (startfast = '1') then
				slowaccess <= '0';
			elsif (falling_edge(extclkdel)) then
				slowaccess <= slowdetected;
			end if;
		end process DoSlowAccess;

		DoSlowDone: process(extclk, startfast, slowaccess)
		begin
			if (startfast = '1') then
				slowdone <= '0';
			elsif (falling_edge(extclk)) then
				-- wait for slow access to finish
				if (rdy = '1') then
					slowdone <= slowaccess;
				end if;
			end if;
		end process DoSlowDone;

		DoFakeClock: process(slowdetected, extclkdel, slowaccess, reset)
		begin
			if (reset='0' or (extclkdel = '1' and slowaccess = '1')) then
				fakeclk <= '0';
			elsif (rising_edge(slowdetected)) then
				fakeclk <= '1';
			end if;
		end process DoFakeClock;

		DoSlowDetect: process(cpuclk, reset, isslow, slowdone)
		begin
			if (reset = '0' or slowdone = '1') then
				-- reset
				slowdetected <= '0';
			elsif rising_edge(cpuclk) then
				slowdetected <= isslow;
			end if;
		end process DoSlowDetect;

end Behavioral;

