----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    23:06:54 02/03/2010 
-- Design Name: 
-- Module Name:    PET816 - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 1.0A - First revision
-- Additional Comments: ----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-- use IEEE.numeric_std.all;
-- use ieee.std_logic_unsigned.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity PET816 is
	Port( 			
			fastclkin : in STD_LOGIC;		-- 50MHz in
			
			-- PET interface

			phi0 : in STD_LOGIC;			-- PET Phi0 in
			phi2 : in STD_LOGIC;			-- PET Phi2 in
			rdy : in STD_LOGIC;
			reset : in STD_LOGIC;
			
			norom : out STD_LOGIC;		-- switch off PET ROM when set to '0'
			rnw : out STD_LOGIC;
			
			-- 65816 interface

			cpuaddrin_64k : in STD_LOGIC_VECTOR (23 downto 16);
			cpuaddrin_2k : in STD_LOGIC_VECTOR (15 downto 11);
			cpuaddr_ismyio : in STD_LOGIC;		-- when 0 then addr bits 4 to 11 are all 1
			cpuaddrin_16 : in STD_LOGIC_VECTOR (3 downto 0);
			cpudata : inout STD_LOGIC_VECTOR (7 downto 0);
			
			cpuvpa : in STD_LOGIC;
			cpuvda : in STD_LOGIC;
			cpuvp : in STD_LOGIC;
			cpurnw : in STD_LOGIC;
			
			cpuclk : out STD_LOGIC;
			
			-- control
			
			nslowbusen : out STD_LOGIC;			-- when 0, pass CPU address bus to PET
			nslowbusclr	:out std_logic;			-- when 0, addr PET addr is set to 0
			nslowdataen : out STD_LOGIC;			-- when 0, pass CPU data bus to PET
			latchen: out STD_LOGIC;					-- '573 latch control (1=passthrough, 0=latch)
			
			-- config
			bootromin: in STD_LOGIC;				-- when 1 then jumper is set to enable boot rom
			
			-- fast memory interface (2 RAM modules)
			
			nramcs : out STD_LOGIC_VECTOR (1 downto 0);
			
			-- bootrom select, in-system-programming
			nromcs : out STD_LOGIC;
			nromwe : out STD_LOGIC;
			
			-- external logic:
			-- phi1 <= not(phi0);
			-- phi2 <= not(phi1);
			-- nramoe <= not(cpurnw);
			-- cpuaddr_ismyio <= not( cpuaddrin(11 downto 4) = "1111111" );
			
			diag : out STD_LOGIC
	);
			
end PET816;

architecture Behavioral of PET816 is

	   	--signal init_slowonly : std_logic := '1';
	   	signal init_bogusok : std_logic := '1';
           	signal init_clklatch : STD_LOGIC_VECTOR (3 downto 0) := "1001";	-- 2.5MHz
--           	signal init_clklatch : STD_LOGIC_VECTOR (3 downto 0) := "0101";	-- 4.2MHz
--           	signal init_clklatch : STD_LOGIC_VECTOR (3 downto 0) := "0010"; -- 8.3MHz
--           	signal init_clklatch : STD_LOGIC_VECTOR (3 downto 0) := "0001"; --12.5MHz

		-- internal state signals
		signal slowonly: std_logic := '1';					-- when 1 then only slow access is allowed
		signal bogusok: std_logic := '1';					-- when 1 then bogus accesses are visible on bus
		signal slow64k: std_logic := '1';					-- when 1 low 64k are mapped to PET (otherwise I/O, video only)
																		--	writes are passed through to fast RAM (with slow access)
		signal bootrom: std_logic := '0';					-- when 1, map expansion ROM to low 64k; initialied to jumper
		signal prgrom: std_logic := '0';						-- when 1, allow writes to ROM (in system programming)
		signal wprotect: std_logic_vector (1 downto 0);	-- when 1, write protect fast RAM map
		
		-- master timing
		signal fastclkbydiv	:std_logic := '0';					-- fast clock divided by clkcount
		signal fastclkby2		:std_logic := '0';				-- fastclkin divided by 2
		signal clklatch		:std_logic_vector (3 downto 0);
		signal slowclk 		:std_logic;
		
		signal extclkdel: std_logic := '0';
		
		-- state engine
		signal slowaccess: std_logic := '0';
		signal nslowaccess: std_logic := '0';
		--signal slowdetected: std_logic := '0';
		signal startfast: std_logic := '0';
		signal fakeclk: std_logic := '0';					-- clk signal between slowdetected and actual slow access
		
		signal cpuclkint: std_logic := '0';
		
		signal validaddr: std_logic := '0';					-- vpa/vda/vp analysis
		
		signal isio: std_logic := '0';			-- 1 when the I/O area is selected (valid together with islow64k)
		signal isvideo: std_logic := '0';		-- 1 when the video area is selected (valid together with islow64k)
		signal islow64k: std_logic := '0';		-- 1 when the lowest 64k are selected
		signal isorig: std_logic := '0';			-- 1 when the original system is being addressed
		signal isorigdel: std_logic := '0';			-- 1 when the original system is being addressed
	
	signal clocked_rnw	:std_logic := '0';
	signal busaccess	:std_logic := '0';		-- 1 when bus is accessed via $Exxxxx

		component ClockDivider
			Port ( 
			  fastclkin : in  STD_LOGIC;
           fastclkby2 : out  STD_LOGIC;
           fastclkbydiv : out  STD_LOGIC;
			  divenable : in STD_LOGIC;
           clklatch : in  STD_LOGIC_VECTOR (3 downto 0);
			  reset : in STD_LOGIC
			);
		end component;

		component ClockShift
			Port (
				fastclkby2 : in std_logic;
				phi : in std_logic;
				phidelayed : out std_logic;
				pulser : out std_logic;
				pulsew : out std_logic;
				pulsec : out std_logic
			);
		end component;
		
		component ControlRegister
		  Port ( data : inout  STD_LOGIC_VECTOR (7 downto 0);
           clk : in  STD_LOGIC;
			  sel : in STD_LOGIC;
			  rnw : in STD_LOGIC;
           reset : in  STD_LOGIC;
           bootromin : in  STD_LOGIC;
           wprotect : out  STD_LOGIC_VECTOR (1 downto 0);
           bogusok : out  STD_LOGIC;
           bootrom : out  STD_LOGIC;
           prgrom : out  STD_LOGIC;
			  slow64k : out STD_LOGIC;
			  slowonly : out STD_LOGIC;
           clklatch : out  STD_LOGIC_VECTOR (3 downto 0);

	   	init_slowonly : in std_logic;
	   	init_bogusok : in std_logic;
           	init_clklatch : in  STD_LOGIC_VECTOR (3 downto 0)
		  );
		end component;

		signal ctrlsel : STD_LOGIC;

		component BoardMemSel
		  Port ( cpuclk : in  STD_LOGIC;
           addr_64k : in  STD_LOGIC_VECTOR (23 downto 16);
           addr_8k : in  STD_LOGIC_VECTOR (15 downto 13);
			  isorig : in std_logic;
           validaddr : in  STD_LOGIC;
           cpurnw : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           prgrom : in  STD_LOGIC;
           bootrom : in  STD_LOGIC;
           wprotect : in  STD_LOGIC_VECTOR (1 downto 0);
			  
			  nramcs : out std_logic_vector(1 downto 0);
			  nromcs : out std_logic;
			  nromwe : out std_logic
			);
		end component;

		signal nromcsint : std_logic;
		
		component ClockSync 
			Port ( 
			  slowaccessout : out  STD_LOGIC;
			  fakeclk : out std_logic;
			  cpuclk : in std_logic;
           startfast : in  STD_LOGIC;
           extclkdel : in  STD_LOGIC;
           extclk : in  STD_LOGIC;
			  rdy : in std_logic;
			  reset : in std_logic;
			  isslow : in std_logic
		);
		end component;

		signal isslow : std_logic;
		signal romaccess : std_logic;
		signal validaddrdel : std_logic;
		
		signal startfastr 	:std_logic;
		signal startfastw		:std_logic;
		signal pulseclr		:std_logic;
		
begin
		-----------------------------------------------------------------------------------------------
		-- system specific part

		DoPetIO: process(cpuaddrin_2k, cpuaddrin_64k, slow64k, bootrom)
		begin
			if (cpuaddrin_64k = "00000000") then
				islow64k <= '1';
			else
				islow64k <= '0';
			end if;

			-- second uppermost 1MByte is place for mapped original bus address space (here 16 times 64k)
			if (cpuaddrin_64k(23 downto 20) = "1110") then
				busaccess <= '1';
			else
				busaccess <= '0';
			end if;

			-- uppermost 1MByte is system ROM area
			if (cpuaddrin_64k(23 downto 20) = "1111") then
				romaccess <= '1';
			else
				romaccess <= '0';
			end if;

			if (cpuaddrin_2k = "11101") then 	-- $XXe800-$XXefff
				isio <= '1';
			else
				isio <= '0';
			end if;

			if (cpuaddrin_2k(15 downto 12) = "1000") then 	-- $XX8xxx
				isvideo <= '1';
			else
				isvideo <= '0';
			end if;

		end process;

		-- isorig must be computed outside of DoPetIO to catch up with changes in islow64k etc
		-- enables bus interface
		isorig <= (islow64k and isvideo) 
			  or (islow64k and isio)
			  or busaccess
			  or (islow64k and slow64k and not(bootrom));

		-----------------------------------------------------------------------------------------------
		-- master timing 
	
		nslowaccess <= not(slowaccess);

		Mytiming: ClockDivider port map (
				fastclkin,
				fastclkby2,
				fastclkbydiv,
				nslowaccess,
				clklatch,
				reset
				);
			
		slowclk <= phi2;
		--phi2 <= slowclk;
		
		-----------------------------------------------------------------------------------------------
		-- bus timing generation
		-- pass phi1 through shift register by fastclock (50MHz)
		-- @50MHz -> @25MHz fastclkb2, each step is 40ns
		
		MyClock: ClockShift port map (
				fastclkby2,
				slowclk,
				extclkdel,
				startfastr,
				startfastw,
				pulseclr
				);
				
		-----------------------------------------------------------------------------------------------
		-- cpu bus timing

	 	RnwDel: process(reset, cpuclkint, cpurnw)
		begin
			if (reset = '0') then
				clocked_rnw <= '1';
				isorigdel <= '0';
				validaddrdel <= '0';
			elsif(rising_edge(cpuclkint)) then
				clocked_rnw <= cpurnw;
				isorigdel <= isorig;
				validaddrdel <= validaddr;
			end if;
		end process;

		-- slow memory: orig mem and slow access (here: all negative logic)
		nslowbusen <= (slowaccess and isorigdel and validaddrdel);
		nslowbusclr <= not(pulseclr);
		-- slow memory; same as nslowbusen, but disable driver when reading own register
		nslowdataen <= not(slowaccess and isorigdel and validaddrdel) or ctrlsel;
		-- slow memory r/-w
		rnw <= clocked_rnw or not (isorigdel) or not(slowaccess) or not(validaddrdel);
		-- switch off original ROMs when 0, 
		norom <= slowaccess and isorigdel and validaddrdel;
		
		-- CPU clock, syntesised
		cpuclkint <= 
				(fastclkbydiv and not(slowaccess or fakeclk))		-- fast access
				or (fakeclk)
				--or ((slowclk or not(clocked_rnw)) and slowaccess);	-- slow access
				or slowaccess;
				
		cpuclk <= cpuclkint;
		latchen <= not(cpuclkint);

		diag <= fakeclk;

		validaddr <= cpuvpa or cpuvda;
		
		-----------------------------------------------------------------------------------------------

		isslow <= (validaddr or bogusok) and ( slowonly or romaccess or isorig );
		startfast <= (clocked_rnw and startfastr) or (not(clocked_rnw) and startfastw);
		
		MyClockSync: ClockSync port map (
			  slowaccess,
			  fakeclk,
			  cpuclkint,
           startfast,
           extclkdel,
           slowclk,
			  rdy,
			  reset,
			  isslow
		);
		
		-----------------------------------------------------------------------------------------------
		-- CPU access to config registers

		ctrlsel <= islow64k and isio and not(cpuaddr_ismyio) and 
						cpuaddrin_16(3) and cpuaddrin_16(2) and cpuaddrin_16(1) and cpuaddrin_16(0);
		
		MyControl : ControlRegister port map (
			cpudata,
			cpuclkint,
			ctrlsel,
			cpurnw,
			reset,
			bootromin,
			wprotect,
			bogusok,
			bootrom,
			prgrom,
			slow64k,
			slowonly,
			clklatch,

			'0',	-- init_slowonly,
			init_bogusok,
			init_clklatch
		);
		
		-----------------------------------------------------------------------------------------------
		-- memory select lines

		--slowdetected <= slowaccess or fakeclk;
		
		memsel : BoardMemSel port map (
			cpuclkint,
			cpuaddrin_64k,
			cpuaddrin_2k(15 downto 13),
			isorig,
			validaddr,
			cpurnw,
			reset,
			prgrom,
			bootrom,
			wprotect,
			
			nramcs,
			nromcsint,
			nromwe
		);

		nromcs <= nromcsint or not(slowclk and slowaccess);
		
end Behavioral;

