library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
 
entity pet816_tb is
end entity;
	 
architecture TB of pet816_tb is
	 
    component PET816
       port( 
                        fastclkin : in STD_LOGIC;               -- 50MHz in

                        -- PET interface

                        phi0 : in STD_LOGIC;                    -- PET Phi1 in
                        rdy : in STD_LOGIC;
                        reset : in STD_LOGIC;

                        norom : out STD_LOGIC;          -- switch off PET ROM when set to '0'
                        rnw : out STD_LOGIC;

                        -- 65816 interface

                        cpuaddrin_64k : in STD_LOGIC_VECTOR (23 downto 16);
                        cpuaddrin_2k : in STD_LOGIC_VECTOR (15 downto 11);
                        cpuaddr_ismyio : in STD_LOGIC;          -- when 0 then addr bits 4 to 11 are all 1
                        cpuaddrin_16 : in STD_LOGIC_VECTOR (3 downto 0);
                        cpudata : inout STD_LOGIC_VECTOR (7 downto 0);

                        cpuvpa : in STD_LOGIC;
                        cpuvda : in STD_LOGIC;
                        cpuvp : in STD_LOGIC;
                        cpurnw : in STD_LOGIC;

                        cpuclk : out STD_LOGIC;

                        -- control

                        nslowbusen : out STD_LOGIC;                     -- when 0, pass CPU address bus to PET
                        nslowdataen : out STD_LOGIC;                    -- when 0, pass CPU data bus to PET
                        latchen: out STD_LOGIC;                                 -- '573 latch control (1=passthrough, 0=latch)

                        -- config
                        bootromin: in STD_LOGIC;                                -- when 1 then jumper is set to enable boot rom

                        -- fast memory interface (2 RAM modules)

                        nramcs : out STD_LOGIC_VECTOR (1 downto 0);

                        -- bootrom select, in-system-programming
                        nromcs : out STD_LOGIC;
                        nromwe : out STD_LOGIC;

                        -- external logic:
                        -- phi1 <= not(phi0);
                        -- phi2 <= not(phi1);
                        -- nramoe <= not(cpurnw);
                        -- cpuaddr_ismyio <= not( cpuaddrin(11 downto 4) = "1111111" );

                        diag : out STD_LOGIC

	 	);
    end component;

    signal reset:   		std_logic;
    signal clk:     		std_logic;
    signal petphi0:    		std_logic;
				
    signal petrdy:		std_logic := '1';
    signal petnorom:		std_logic;
    signal petrnw:		std_logic;
       
    signal cpuaddr:		std_logic_vector(23 downto 0);
    signal nismyio:		std_logic := '1';
    signal cpudata:		std_logic_vector(7 downto 0);

    signal cpuvda:		std_logic := '1';
    signal cpuvpa:		std_logic := '1';
    signal cpuvp:		std_logic := '1';
    signal cpurnw:		std_logic := '1';
    signal cpuclk:		std_logic;

    signal nslowbusen:		std_logic;
    signal nslowdataen:		std_logic;
    signal latchen:		std_logic;

    signal bootromin:		std_logic := '0';

    signal nramcs:		std_logic_vector(1 downto 0);
    signal nromcs:		std_logic := '0';
    signal nromwe:		std_logic := '0';

    signal diag:		std_logic;

begin
     
    core: PET816 port map (
    		            clk,

                            -- PET interface

                            petphi0,
                            petrdy,
                            reset,

                            petnorom,
                            petrnw,

                            -- 65816 interface

                            cpuaddr(23 downto 16),
                            cpuaddr(15 downto 11),
			    nismyio,
                            cpuaddr(3 downto 0),

                            cpudata,

                            cpuvpa,
                            cpuvda,
                            cpuvp,
                            cpurnw,

                            cpuclk,

                            -- control

                            nslowbusen,
                            nslowdataen,
                            latchen,

                            -- config
                            bootromin,

                            -- fast memory interface (2 RAM modules)

                            nramcs,

                            -- bootrom select, in-system-programming
                            nromcs,
                            nromwe,

                            diag

    );
	 
    	process
    	begin
		clk <= '0';
		wait for 11 ns;
		clk <= '1';
		wait for 10 ns;
    	end process;

	process
	begin
		petphi0 <= '0';
		wait for 500 ns;
		petphi0 <= '1';
		wait for 500 ns;
	end process;

	process
	begin
		reset <= '0';
		wait for 155 ns;
		reset <= '1';
		wait for 10000000 ns;
	end process;

end;
